from cuttlefish.actions import CuttleAction
from cuttlefish.events import CuttleEvent
from cuttlefish.plugins import CuttlePlugin
from cuttlefish.params import StringParam, BoolParam, SelectAppParam, FolderParam, FileParam

from gi.repository import Wnck
import subprocess
import os
import os.path
import signal

class RunApplication(CuttleAction, CuttlePlugin):
	NAME = 'Start application'
	DESCP = 'Starts an application'
	CATEGORY = 'Applications'
	PARAMS = {
		'path' : 'firefox',
		'wait' : False
	}

	class Editor(CuttlePlugin.Editor):
		ORDER = ['path', 'wait']

		def begin(self):
			return {
				'path': SelectAppParam('Application'),
				'wait': BoolParam('Wait until application exits')
			}

	def __init__(self):
		CuttleAction.__init__(self)
		CuttlePlugin.__init__(self)
		self._proc = None
		
	def execute(self):
		argv = self._params['path'].split(' ')
		self._proc = subprocess.Popen(argv)
		
		if self._params['wait']:
			self._proc.wait()

	def interrupt(self):
		if self._params['wait']:
			self._proc.kill()


class RunApplicationEx(CuttleAction, CuttlePlugin):
	NAME = 'Start application (advanced mode)'
	DESCP = 'Starts any executable application with extra parameters'
	CATEGORY = 'Applications'
	PARAMS = {
		'path': '/usr/bin/firefox', 
		'params': '', 
		'wdir': '', 
		'wait' : False
	}

	class Editor(CuttlePlugin.Editor):
		ORDER = ['path', 'params', 'wdir', 'wait']

		def begin(self):
			return {
			'path': FileParam('Executable'),
			'params': StringParam('Parameters'),
			'wdir': FolderParam('Working directory'),
			'wait': BoolParam('Wait until application exits')
		}

	def __init__(self):
		CuttleAction.__init__(self)
		CuttlePlugin.__init__(self)
		self._proc = None
		
	def execute(self):
		argv = (self._params['path'] + ' ' + self._params['params']).split(' ')
		wdir = self._params['wdir']
		if wdir == '':
			wdir = None
		self._proc = subprocess.Popen(argv,cwd=wdir)
		
		if self._params['wait']:
			self._proc.wait()

	def interrupt(self):
		if self._params['wait']:
			self._proc.kill()
		


class KillApplication(CuttleAction, CuttlePlugin):
	NAME = 'Stop an application'
	DESCP = 'Kill all instances of an application'
	CATEGORY = 'Applications'
	PARAMS = {
		'name' : 'firefox'
	}

	class Editor(CuttlePlugin.Editor):
		def begin(self):
			return {
				'name': SelectAppParam('Application')
			}
		
	def __init__(self):
		CuttleAction.__init__(self)
		CuttlePlugin.__init__(self)

	def execute(self):
		p = subprocess.Popen(['ps', '-A', 'h'], stdout=subprocess.PIPE)
		out, err = p.communicate()
		for line in out.splitlines():
			if self._params['name'].upper() in line.upper():
				pid = int(line.split(None, 1)[0])
				os.kill(pid, signal.SIGKILL)



def _pid_to_name(pid):
	ps = subprocess.Popen(['ps', '-p', str(pid),'-o', 'cmd', 'h'], stdout=subprocess.PIPE)
	psname = ps.communicate()[0][:-1].strip()
	# remove path and additional call parameters
	psname = os.path.basename(psname).split(' ', 1)[0]
	return psname



class ApplicationStart(CuttleEvent, CuttlePlugin):
	NAME = 'Application starts'
	DESCP = "React when a specified application starts"
	CATEGORY = 'Applications'
	PARAMS =  {
		'application' : 'firefox',
		'firstOnly' : False
	}
	
	class Editor(CuttlePlugin.Editor):
		ORDER = ['application', 'firstOnly']

		def begin(self):
			return {
				'application' : SelectAppParam('Application'), 
				'firstOnly' : BoolParam('Trigger only the first instance')
			}
		
	def __init__(self):
		CuttleEvent.__init__(self)
		CuttlePlugin.__init__(self)
		self._screen = Wnck.Screen.get_default()

	def setup(self):
		self._screen.force_update()
		self._hid = self._screen.connect("window-opened", self.on_window_opened)
		
	def teardown(self):
		self._screen.disconnect(self._hid)
	
	def on_window_opened(self, screen, window):
		app = window.get_application()
		psname = _pid_to_name(app.get_pid())
		
		if os.path.basename(self._params['application']).upper() == psname.upper():
			if self._params['firstOnly']:
				isFirst = True
				for wnd in self._screen.get_windows():
					app2 = wnd.get_application()
					if app.get_name() == app2.get_name() and app2.get_pid() != app.get_pid():
						isFirst = False
						break
				if isFirst:
					self.trigger()
			else:
				self.trigger()



class ApplicationEnd(CuttleEvent, CuttlePlugin):
	NAME = 'Application stops'
	DESCP = "React when a specified application finishes"
	CATEGORY = 'Applications'
	PARAMS =  {
		'application' : 'firefox',
		'lastOnly' : False
	}
	
	class Editor(CuttlePlugin.Editor):
		ORDER = ['application', 'lastOnly']

		def begin(self):
			return {
				'application' : SelectAppParam('Application'), 
				'lastOnly' : BoolParam('Trigger only the last instance')
			}
		
	def __init__(self):
		CuttleEvent.__init__(self)
		CuttlePlugin.__init__(self)
		self._screen = Wnck.Screen.get_default()


	def setup(self):
		self._pids = {}
		self._screen.force_update()
		# ugly code, but I only need to call ps once :)
		pids = [str(w.get_application().get_pid()) for w in self._screen.get_windows() if w.get_application().get_pid() > 0]
		ps = subprocess.Popen(('ps -p %s -o pid,cmd h' % ','.join(pids)).split(' '), stdout=subprocess.PIPE)
		cmds = [cmd.strip().split(' ', 1) for cmd in ps.communicate()[0][:-1].split('\n')]
		self._pids = { int(cmd[0]) : os.path.basename(cmd[1]).split(' ', 1)[0] for cmd in cmds}
		
		self._hid_open = self._screen.connect("window-opened", self.on_window_opened)
		self._hid_close = self._screen.connect("application-closed", self.on_window_closed)
		
	def teardown(self):
		self._screen.disconnect(self._hid_open)
		self._screen.disconnect(self._hid_close)
		self._pids = {}


	def on_window_opened(self, screen, window):
		pid = window.get_application().get_pid()
		if pid > 0:
			psname = _pid_to_name(pid)		
			self._pids[pid] = psname

	
	def on_window_closed(self, screen, app):
		pid = app.get_pid()
		if pid in self._pids:
			psname = self._pids[pid]
			del self._pids[pid]

			if os.path.basename(self._params['application']).upper() == psname.upper():
				if self._params['lastOnly']:
					isLast = True
					for wnd in self._screen.get_windows():
						app2 = wnd.get_application()
						if app.get_name() == app2.get_name() and app2.get_pid() != app.get_pid():
							isLast = False
							break
					if isLast:
						self.trigger()
				else:
					self.trigger()
